// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:
#ifndef SPHERE_HH
#define SPHERE_HH

#include <dune/udg/2dtriangulation/innerdomain.hh>

#include <limits>

namespace Dune {
  namespace UDG  {

    template<int dim, class ct>
    class Sphere : public InnerDomain<dim, ct> {
      Dune::FieldVector< ct, dim > c;
      ct r;
      inline ct sqr(ct x) const { return x*x; }
    public:
      Sphere() : c(1e99), r(0) {};
      Sphere(const Dune::FieldVector<ct,dim> & _c, ct _r) : c(_c), r(_r) {};
      Sphere(ct cx, ct cy, ct _r) : r(_r) { c[0] = cx; c[1] = cy; };
      Sphere(const Sphere & s) : c(s.c), r(s.r) {};
      virtual ~Sphere() {};
      virtual void polynom(const Dune::FieldVector<ct,dim> & A,
                           const Dune::FieldVector<ct,dim> & B, int & side,
                           Dune::FieldVector<ct,3> & a,
                           Dune::FieldVector<ct,3> & b) const;
      virtual void specialPoints(std::vector<Dune::FieldVector<ct,dim> > & points) const {
        points.clear();
        points.push_back(c);
        Dune::FieldVector<ct,dim> e = c;
        e[1] = c[1]+r;
        points.push_back(e);
        e[1] = c[1]-r;
        points.push_back(e);
        e = c;
        e[0] = c[0]+r;
        points.push_back(e);
        e[0] = c[0]-r;
        points.push_back(e);
      };
      virtual const Dune::FieldVector<ct,dim> & begin() const { return c; };
      virtual const Dune::FieldVector< ct, dim >
      normal (const Dune::FieldVector< ct, dim > & p) const;
      virtual bool isInside(const Dune::FieldVector<ct,dim> & p) const {
        return ( sqr(p[0]-c[0]) + sqr(p[1]-c[1]) - sqr(r) < 0 );
      };
      /*
           (A+e)^2 + (B+e)^2 <= r^2
         => A^2 + B^2 - r^2 <= 2Ae + 2Be + O(e^2)
         ~> A^2 + B^2 - r^2 <= 2Re + 2Re + Re
         => A^2 + B^2 - r^2 <= 5Re
       */
      virtual bool isInsideInc(const Dune::FieldVector<ct,dim> & p) const {
        double eps = 5*r*std::numeric_limits<ct>::epsilon();
        return ( (p-c).two_norm2() - sqr(r) <= eps );
      };
      virtual void getIntersections(const Line<dim,ct>& l, std::vector<Dune::FieldVector<ct,dim> >& points) const;
      virtual void print(std::ostream& s) const {
        s << "Sphere(" << c << "," << r << ")";
      }
      ///// additional interface
      const Dune::FieldVector<ct,dim> & center() const { return c; }
      const ct & radius() const { return r; }
    };

  } // end namespace UDG
} // end namespace DUNE

#endif
