#ifndef DUNE_UDG_COMMON_QUADRATURERULES_HH
#define DUNE_UDG_COMMON_QUADRATURERULES_HH

#include <dune/geometry/quadraturerules.hh>

#include <dune/udg/vtriangulation.hh>

namespace Dune {
  namespace UDG {
    namespace Impl {
      template<typename VST>
      class VSTQuadratureRule : public QuadratureRule<typename VST::ctype,VST::Grid::dimension>
      {
      public:
        VSTQuadratureRule(const VST& vst, const typename VST::Entity& e, const int intorder,
                          const QuadratureType::Enum quadrature_type = QuadratureType::GaussLegendre)
        {
          using ctype = typename VST::ctype;
          const auto dim = VST::Grid::dimension;
          auto entityParts = typename VST::EntityPartList();
          vst.create_entity_parts(e, entityParts);
          for(const auto& ep : entityParts)
          {
            //mapping to father entity
            auto& geoEntity = ep->geometryInEntity();
            // generate a quadrature rule for the current entity part
            auto& localRule = QuadratureRules<ctype,dim>::rule(geoEntity.type(),intorder,quadrature_type);
            // transform the quadrature points to coordinates of the father
            // entity and add them to the main quadrature rule,
            // adjust the weights accordingly
            for(const auto& qp : localRule)
              this->push_back(QuadraturePoint<ctype,dim>(geoEntity.global(qp.position()),
                                                         geoEntity.integrationElement(qp.position()) * qp.weight()));
          }
        }
      };
    } //namespace Impl

    /** \brief Generates a quadrature rule for a
        subtriangulated entity
        \tparam VST The subtriangulation type
        \param vst The subtriangulation being used
        \param e The entity the quadrature rule will be generated for
        \param order The desired integration order on the snippets
        \param quadrature_type The desired quadrature type being used on the snippets
    */
    template<typename VST>
    QuadratureRule<typename VST::ctype, VST::Grid::dimension>
    quadratureRule(const VST& vst, const typename VST::Entity& e, const std::size_t order,
                        const QuadratureType::Enum quadrature_type = QuadratureType::GaussLegendre)
    { return Impl::VSTQuadratureRule<VST>(vst,e,order,quadrature_type); }

  } //namespace UDG
} //namespace Dune
#endif
