// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:
#ifndef DUNE_BINARY_HH
#define DUNE_BINARY_HH

#include <iostream>
#include <dune/common/version.hh>
#if !DUNE_VERSION_NEWER(DUNE_COMMON,2,4)
#include <dune/common/static_assert.hh>
#endif
#include <dune/common/typetraits.hh>

template<class T>
class Binary
{
private:
  T* ptr;
public:
  Binary () {}
  Binary (T & d) {
    ptr = & d;
  }
  Binary & operator = (T & d) {
    ptr = & d;
    return *this;
  }
  operator T ()
  {
    return *ptr;
  }
  operator const T () const
  {
    return *ptr;
  }
  char* data() {
    return reinterpret_cast<char*>(ptr);
  }
  const char* data() const {
    return reinterpret_cast<const char*>(ptr);
  }
};

template<class T>
class Binary<const T>
{
  // Binary<const T> is not allowed, to avoid problems with temporary objects
};

template <class T>
Binary<T> binary(T & t)
{
#if DUNE_VERSION_NEWER(DUNE_COMMON,2,4)
  static_assert(! std::is_const<T>::value,
                     "binary of temporary is not allowed");
#else
  dune_static_assert(! std::is_const<T>::value,
                     "binary of temporary is not allowed");
#endif
  return Binary<T>(t);
}

template <class T>
Binary<T> binary(const T & t)
{
#if DUNE_VERSION_NEWER(DUNE_COMMON,2,4)
  static_assert(! std::is_const<const T>::value,
                     "binary of temporary is not allowed");
#else
  dune_static_assert(! std::is_const<const T>::value,
                     "binary of temporary is not allowed");
#endif
  return Binary<const T>(t);
}

template<class T>
std::istream & operator >> (std::istream & s, Binary<T> & d)
{
  s.read(d.data(), sizeof(T));
  return s;
}

template<class T>
std::ostream & operator << (std::ostream & s, const Binary<T> & d)
{
  s.write(d.data(), sizeof(T));
  return s;
}

#endif
