#ifndef DUNE_UDG_CUTCELLINTERFACEBASE_HH
#define DUNE_UDG_CUTCELLINTERFACEBASE_HH

#include <vector>
#include <cmath>

namespace Dune
{
  namespace UDG
  {
    namespace CutCellInterfaceBaseDetail
    {
      template <int dim, class Snippet>
      void checkSnippet(const Snippet& snippet)
      {
        for (int i = 0; i < snippet.globalGeometry().corners(); ++i) {
          for (unsigned int d = 0; d < dim; ++d) {
            if (std::isnan(snippet.globalGeometry().corner(i)[d])) {
              DUNE_THROW(Dune::Exception, "corner is nan");
            }
          }
        }
        if (snippet.globalGeometry().volume() < 1e-32) {
          DUNE_THROW(Dune::Exception, "volume is 0");
        }
      }
    }
    template <class GV, class LGV, class Snippet>
    class CutCellInterfaceBase
    {
    public:
      using const_iterator = typename std::vector<Snippet>::const_iterator;

      const_iterator begin() const
      {
        return snippets_.begin();
      }

      const_iterator end() const
      {
        return snippets_.end();
      }

      bool empty() const
      {
        return snippets_.empty();
      }

      std::size_t size() const
      {
        return snippets_.size();
      }

      void addSnippet(const Snippet& snippet)
      {
#ifndef NDEBUG
        CutCellInterfaceBaseDetail::checkSnippet<GV::dimension>(snippet);
#endif
        snippets_.push_back(snippet);
      }

      void addSnippet(Snippet&& snippet)
      {
#ifndef NDEBUG
        CutCellInterfaceBaseDetail::checkSnippet<GV::dimension>(snippet);
#endif
        snippets_.push_back(std::move(snippet));
      }

      virtual ~CutCellInterfaceBase()
      {
      }

    private:
      std::vector<Snippet> snippets_;
    };
  }
}

#endif // DUNE_UDG_CUTCELLINTERFACEBASE_HH
