#ifndef DUNE_UDG_EXTERNALINTERFACEAREAMAP_HH
#define DUNE_UDG_EXTERNALINTERFACEAREAMAP_HH

#include <map>
#include <dune/geometry/quadraturerules.hh>
#include <dune/udg/simpletpmctriangulation/domainconfiguration.hh>
#include <dune/udg/simpletpmctriangulation/externalinterfacesubtriangulation.hh>

namespace Dune
{
  namespace UDG
  {
    template <class GV, class LGV>
    class ExternalInterfaceAreaMap
    {
    public:
      using FundamentalIntersection = typename GV::Intersection;
      using ctype = typename GV::ctype;
      enum { dim = GV::dimension };

      ExternalInterfaceAreaMap(const GV& fundamentalGridView,
          const ExternalInterfaceSubTriangulation<GV, LGV>& subTriangulation,
          const DomainConfiguration<GV, LGV>& domainConfiguration)
          : numberOfDomains_(domainConfiguration.size())
          , areas_(
                domainConfiguration.size() * subTriangulation.fundamentalElement().subEntities(1),
                0.0)
      {
        for (const Domain& domain : domainConfiguration.domains()) {
          for (const ExternalCutCellInterface<GV, LGV>& ei :
              subTriangulation.externalCutCellInterfaces(domain.index())) {
            for (const ExternalInterfaceSnippet<GV, LGV>& snippet : ei) {
              auto& currentArea = areas_[linear(snippet.codim1IndexInInside(), domain.index())];
              const auto& rule
                  = Dune::QuadratureRules<ctype, dim - 1>::rule(snippet.globalGeometry().type(), 1);
              for (const auto& qp : rule) {
                currentArea
                    += qp.weight() * snippet.globalGeometry().integrationElement(qp.position());
              }
            }
          }
        }
      }

      ctype area(int codim1IndexInInside, const Domain& domain) const
      {
        return areas_[linear(codim1IndexInInside, domain.index())];
      }

    private:
      std::size_t linear(int codim1index, std::size_t domainIndex) const
      {
        return codim1index * numberOfDomains_ + domainIndex;
      }

      std::size_t numberOfDomains_;
      std::vector<ctype> areas_;
    };
  }
}

#endif // DUNE_UDG_EXTERNALINTERFACEAREAMAP_HH
