#ifndef DUNE_UDG_EXTERNALINTERFACESUBTRIANGULATION_HH
#define DUNE_UDG_EXTERNALINTERFACESUBTRIANGULATION_HH

#include <vector>
#include <dune/grid/common/rangegenerators.hh>
#include <dune/udg/simpletpmctriangulation/domainconfiguration.hh>
#include <dune/udg/simpletpmctriangulation/externalcutcellinterface.hh>

namespace Dune
{
  namespace UDG
  {
    template <class GV, class LGV>
    class ExternalInterfaceSubTriangulation
    {
    public:
      using ctype = typename GV::ctype;
      enum { dim = GV::dimension };
      using FundamentalElement = typename GV::template Codim<0>::Entity;

      ExternalInterfaceSubTriangulation(const GV& fundamentalGridView, const LGV& levelSetGridView,
                                        const DomainConfiguration<GV, LGV>& domainConfiguration,
                                        const FundamentalElement& element,
                                        ctype valueTolerance = 1e-8)
          : fundamentalElement_(element)
          , domainToExternalInterfaces_(domainConfiguration.numberOfDomains())
      {
        std::vector<ExternalCutCellInterface<GV, LGV> > externalCutCellInterfaces;
        externalCutCellInterfaces.push_back(ExternalCutCellInterface<GV, LGV>(
            element, fundamentalGridView, levelSetGridView, valueTolerance));

        for (const Interface<LGV>& interface : domainConfiguration.interfaces()) {
          std::vector<ExternalCutCellInterface<GV, LGV> > newExternalInterfaces;
          for (const ExternalCutCellInterface<GV, LGV>& ei : externalCutCellInterfaces) {
            ei.cut(interface, std::back_inserter(newExternalInterfaces));
          }
          externalCutCellInterfaces.swap(newExternalInterfaces);
        }
        for (ExternalCutCellInterface<GV, LGV>& ei : externalCutCellInterfaces) {
          auto domainIt = domainConfiguration.findDomain(ei.interfaceRelativePositions());
          if (domainIt != domainConfiguration.domainsEnd()) {
            domainToExternalInterfaces_[domainIt->index()].push_back(std::move(ei));
          }
        }
      }

      const std::vector<ExternalCutCellInterface<GV, LGV> >& externalCutCellInterfaces(
          std::size_t domainIndex) const
      {
        return domainToExternalInterfaces_[domainIndex];
      }

      const FundamentalElement& fundamentalElement() const
      {
        return fundamentalElement_;
      }

    private:
      FundamentalElement fundamentalElement_;
      std::vector<std::vector<ExternalCutCellInterface<GV, LGV> > > domainToExternalInterfaces_;
    };
  }
}

#endif // DUNE_UDG_EXTERNALINTERFACESUBTRIANGULATION_HH
