#ifndef DUNE_UDG_INTERNALINTERFACESUBTRIANGULATION_HH
#define DUNE_UDG_INTERNALINTERFACESUBTRIANGULATION_HH

#include <dune/udg/simpletpmctriangulation/domainconfiguration.hh>
#include <dune/udg/simpletpmctriangulation/internalcutcellinterface.hh>

namespace Dune
{
  namespace UDG
  {

    template <class GV, class LGV>
    class InternalInterfaceSubTriangulation
    {
    public:
      using ctype = typename GV::ctype;
      enum { dim = GV::dimension };
      using FundamentalElement = typename GV::template Codim<0>::Entity;
      using LevelSetElement = typename LGV::template Codim<0>::Entity;

      InternalInterfaceSubTriangulation(const LGV& levelSetGridView,
                                        const DomainConfiguration<GV, LGV>& domainConfiguration,
                                        const FundamentalElement& element,
                                        ctype valueTolerance = 1e-8)
          : fundamentalElement_(element)
      {
        if (element.isLeaf()) {
          for (const Interface<LGV>& interface : domainConfiguration.interfaces()) {
            InternalCutCellInterface<GV, LGV> internalinterface(
                element, std::make_shared<LevelSetElement>(element), interface, valueTolerance);
            if (!internalinterface.empty()) {
              internalInterfaces_.push_back(internalinterface);
            }
          }
        } else {
          auto maxLevel = levelSetGridView.grid().maxLevel();
          // loop through all child elements in the level set mesh
          for (auto it = fundamentalElement_.hbegin(maxLevel);
               it != fundamentalElement_.hend(maxLevel); ++it) {
            if (!levelSetGridView.indexSet().contains(*it))
              continue;
            auto levelSetElement = std::make_shared<LevelSetElement>(*it);
            for (const Interface<LGV>& interface : domainConfiguration.interfaces()) {
              InternalCutCellInterface<GV, LGV> internalinterface(element, levelSetElement,
                                                                  interface, valueTolerance);
              if (!internalinterface.empty()) {
                internalInterfaces_.push_back(internalinterface);
              }
            }
          }
        }
        // cut all other interfaces into each internalinterface
        for (const Interface<LGV>& interface : domainConfiguration.interfaces()) {
          std::vector<InternalCutCellInterface<GV, LGV> > newInternalInterfaces;
          for (const InternalCutCellInterface<GV, LGV>& ii : internalInterfaces_) {
            if (ii.interface() != interface) {
              ii.cut(interface, std::back_inserter(newInternalInterfaces));
            } else {
              newInternalInterfaces.push_back(ii);
            }
          }
          internalInterfaces_.swap(newInternalInterfaces);
        }
      }

      const std::vector<InternalCutCellInterface<GV, LGV> >& internalCutCellInterfaces() const
      {
        return internalInterfaces_;
      }

      const FundamentalElement& fundamentalElement() const
      {
        return fundamentalElement_;
      }

    private:
      FundamentalElement fundamentalElement_;
      std::vector<InternalCutCellInterface<GV, LGV> > internalInterfaces_;
    };
  }
}

#endif // DUNE_UDG_INTERNALINTERFACESUBTRIANGULATION_HH
