#ifndef DUNE_UDG_VOLUMESNIPPET_HH
#define DUNE_UDG_VOLUMESNIPPET_HH

#include <dune/geometry/multilineargeometry.hh>
#include <dune/udg/simpletpmctriangulation/interface.hh>

namespace Dune
{
  namespace UDG
  {
    /** \brief a snippet is a part of a cut cell
        \tparam GV the type of the fundamental mesh grid view
        \tparam LGV the type of the level set mesh grid view
     */
    template <class GV, class LGV>
    class VolumeSnippet
    {
    public:
      //! \brief type of the fundamental mesh grid view
      using FundamentalGridView = GV;
      using LevelSetGridView = LGV;
      using Element = typename LGV::template Codim<0>::Entity;
      using ctype = typename GV::ctype;
      enum { dim = GV::dimension };
      //! \brief geometry type of this snippet in the fundamental mesh cell
      using GeometryInFundamental = Dune::CachedMultiLinearGeometry<ctype, dim, dim,
          Dune::MultiLinearGeometryTraits<ctype> >;

      VolumeSnippet(std::size_t index, const GeometryInFundamental& geometryInFundamental,
          std::shared_ptr<Element> homeElement)
          : index_(index)
          , geometryInFundamental_(geometryInFundamental)
          , homeElement_(homeElement)
      {
      }

      VolumeSnippet(std::size_t index, Dune::GeometryType type,
          const std::vector<FieldVector<ctype, dim> >& corners,
          std::shared_ptr<Element> homeElement)
          : index_(index)
          , geometryInFundamental_(type, corners)
          , homeElement_(homeElement)
      {
      }

      std::size_t index() const
      {
        return index_;
      }

      const GeometryInFundamental& geometryInFundamental() const
      {
        return geometryInFundamental_;
      }

      std::shared_ptr<Element> homeElement() const
      {
        return homeElement_;
      }

    private:
      std::size_t index_;
      GeometryInFundamental geometryInFundamental_;
      std::shared_ptr<Element> homeElement_;
    };
  }
}

#endif // DUNE_UDG_VOLUMESNIPPET_HH
