#ifndef DUNE_UDGTEST_COMMON_DOMAINS_HH
#define DUNE_UDGTEST_COMMON_DOMAINS_HH

namespace Dune {
  namespace UDGTest {

    template <class GV, class LS>
    struct DomainFactoryInterface
    {
      typedef Dune::UDG::DomainConfiguration<GV,GV> DD;
      virtual DD construct(const GV& gridView, const std::vector<LS>& levelSets) const = 0;
      virtual ~DomainFactoryInterface() {}
    };

    template <class GV, class LS>
    class CrossDomainFactory
      : public DomainFactoryInterface<GV,LS>
    {
    public:
      typedef DomainFactoryInterface<GV,LS> BaseT;
      typedef typename BaseT::DD DD;

      virtual DD construct(const GV& gridView, const std::vector<LS>& levelSets) const
      {
        DD domainConfiguration;

        // construct all the possible domains (that are not empty and make sense here)
        // this depends highly on the ordering of the levelsets, see levelsets.hh
        std::vector<std::string> pos = {"ii","ei","ie"};
        std::vector<std::string> data(pos.size());
        std::vector<std::string> result;

        allDomainConfigurationsRecursive(pos, data, result, pos.size()-1, 0);

        // add domains to the domain configuration
        for (std::size_t i=0; i<result.size(); i++)
	{
          domainConfiguration.addDomain(Dune::UDG::Domain(i, {result[i]}));
	}

        // add interfaces to the domain configuration
        for (std::size_t i=0; i<levelSets.size(); ++i)
        {
          domainConfiguration.addInterface(Dune::UDG::Interface<GV>(i, levelSets[i]));
        }
        return domainConfiguration;
      }

    private:
      // computes all domain configurations using last+1 strings from str and store all those strings in result
      void allDomainConfigurationsRecursive(std::vector<std::string> str,
                                            std::vector<std::string>& data,
                                            std::vector<std::string>& result,
                                            int last,
                                            int index) const
      {
        for (std::size_t i=0; i<str.size(); i++)
        {
          data[index] = str[i];

          if (index == last)
          {
            std::string temp;
            for (auto& e : data)
              temp += e;
            result.push_back(temp);
          }
          else
            allDomainConfigurationsRecursive(str, data, result, last, index+1);
        }
      }
    };

  }
}

#endif // DUNE_UDGTEST_COMMON_DOMAINS_HH
