#ifndef DUNE_UDGTEST_COMMON_LEVELSETS_HH
#define DUNE_UDGTEST_COMMON_LEVELSETS_HH

#include <dune/functions/gridfunctions/gridviewfunction.hh>

namespace Dune
{
  namespace UDGTest
  {
    /**
     * \brief Construct two halfspacelevelsets for each dimension with given
     *        distance and rotated around first axis by phi.
     */
    template <class GV, class OutIterator>
    void constructRotatedCrossLevelSets(
        const GV& gridView, const Dune::FieldVector<typename GV::ctype, GV::dimension>& distances,
        double phi, OutIterator out)
    {
      typedef typename GV::ctype ctype;
      const int dim = GV::dimension;
      using Range = ctype;
      using Domain = Dune::FieldVector<ctype, dim>;
      using LevelSet = Dune::Functions::GridViewFunction<Range(Domain), GV>;
      // define rotation matrix
      Dune::FieldMatrix<ctype, dim, dim> rotation;
      rotation[0][0] = std::cos(phi);
      rotation[0][1] = std::sin(phi);
      if (dim > 2)
        rotation[0][2] = 0;
      rotation[1][0] = -std::sin(phi);
      rotation[1][1] = std::cos(phi);
      if (dim > 2)
        rotation[1][2] = 0;
      if (dim > 2) {
        rotation[2][0] = 0;
        rotation[2][1] = 0;
        rotation[2][2] = 1;
      }
      for (int d = 0; d < dim; ++d) {
        for (int dir = 0; dir < 2; ++dir) {
          Domain offset(0.5);
          offset[d] += (2 * dir - 1) * distances[d] / 2;
          Domain normal(0);
          normal[d] = 2 * dir - 1;
          // rotate normal
          Domain rotatedNormal;
          rotation.mv(normal, rotatedNormal);
          // construct and return level set
          *out++ = Dune::Functions::makeAnalyticGridViewFunction(
              [offset, rotatedNormal](Domain x) {
                x -= offset;
                return x * rotatedNormal;
              },
              gridView);
        }
      }
    }
  }
}

#endif // DUNE_UDGTEST_COMMON_LEVELSETS_HH
