// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:
#include <iostream>
#include <config.h>

#include "dune/udg/common/geometry.hh"

template<int D, int G>
void test(const SubTriangulationGeometry<D,G,double> & g,
  const Dune::FieldVector<double,D> x,
  const double ie,
  const Dune::FieldVector<double,G> global
  )
{
  std::cout << "Coord: " << x << std::endl;
  auto i = g.integrationElement(x);
  auto gb = g.global(x);
  std::cout << "got " << i << "\texpected " << ie
            << "\tdifference " << std::abs(i - ie)
            << std::endl;
  std::cout << "got (" << gb << ")\texpected (" << global
            << ")" << std::endl;
  assert(std::abs(i - ie) < 1e-6);
  assert((gb - global).two_norm() < 1e-6);
}

int main ()
{
  // bilinear basis functions:
  // 1, x, y, x*y
  Dune::FieldMatrix<double, 6, 2> coeffs(0);
  coeffs[0][0] = 10; // x_g = 10 + x
  coeffs[1][0] = 1;
  coeffs[2][0] = 0;
  coeffs[4][0] = 0;
  coeffs[0][1] = 20; // y_g = 20 + y + 2*x*y
  coeffs[1][1] = 0;
  coeffs[2][1] = 1;
  coeffs[4][1] = 2;
  SubTriangulationGeometry<2,2,double> g(Dune::GeometryTypes::cube(2),coeffs);

  test(g, {0.0,0.0}, 1.0, {10.0, 20.0});
  test(g, {0.5,0.5}, 2.0, {10.5, 21.0});
  test(g, {1.0,1.0}, 3.0, {11.0, 23.0});

  // quadratic basis functions:
  // 1, x, x^2
  Dune::FieldMatrix<double, 3, 2> coeffs2(0);
  coeffs2[0][0] = 0; // x_g = x
  coeffs2[1][0] = 1;
  coeffs2[2][0] = 0;
  coeffs2[0][1] = 0; // y_g = x+x^2
  coeffs2[1][1] = 1;
  coeffs2[2][1] = 2;
  SubTriangulationGeometry<1,2,double> l(Dune::GeometryTypes::cube(2),coeffs2);

  test(l, {0.0}, std::sqrt(2.0),  {0.0, 0.0});
  test(l, {0.5}, std::sqrt(10.0), {0.5, 1.0});
  test(l, {1.0}, std::sqrt(26.0), {1.0, 3.0});
}
