#ifndef DUNE_UDGTEST_UTILITY_VTK_HH
#define DUNE_UDGTEST_UTILITY_VTK_HH

namespace Dune {
  namespace UDGTest {

    /**
     * \brief Write a set of intersection parts along with some integer data to vtk in simple
     *        legacy format.
     */
    template <class ctype, int dim, class T>
    void writeVTK(const std::string& filename, const std::string& title, const std::string& dataname,
                  const std::vector<std::pair<std::vector<Dune::FieldVector<ctype,dim> >,T> >& parts)
    {
      // count coordinates
      int count = 0;
      for (int i = 0; i<parts.size(); ++i)
      {
        count += parts[i].first.size();
      }
      // write header
      std::ofstream stream(filename);
      stream << "# vtk DataFile Version 2.0\n"
             << title << "\n"
             << "ASCII\n"
             << "DATASET UNSTRUCTURED_GRID\n"
             << "POINTS " << count << " float\n";
      // write coordinates
      for (int i = 0; i<parts.size(); ++i)
      {
        for (int j = 0; j<parts[i].first.size(); ++j)
        {
          for (int d = 0; d<dim; ++d)
          {
            stream << std::setprecision(20) << (d>0 ? " " : "") << parts[i].first[j][d];
          }
          for (int d = dim; d<3; ++d)
            stream << " " << 0;
          stream << "\n";
        }
      }
      // write elements
      stream << "CELLS " << parts.size() << " " << count+parts.size() << "\n";
      int offset = 0;
      for (int i = 0; i<parts.size(); ++i)
      {
        stream << parts[i].first.size();
        for (int j = 0; j<parts[i].first.size(); ++j)
        {
          stream << " " << offset+j;
        }
        stream << "\n";
        offset += parts[i].first.size();
      }
      stream << "CELL_TYPES " << parts.size() << "\n";
      for (int i = 0; i<parts.size(); ++i)
      {
        int key = 0;
        switch (parts[i].first.size())
        {
        case 2 : key = 3; break; // line
        case 3 : key = 5; break; // triangle
        case 4 : key = 8; break; // square
        }
        stream << key << "\n";
      }
      // write cell index data
      stream << "CELL_DATA " << parts.size() << "\n";
      stream << "SCALARS " << dataname << " float\n";
      stream << "LOOKUP_TABLE default\n";
      for (int i = 0; i<parts.size(); ++i)
      {
        stream << parts[i].second << "\n";
      }
    }

  }
}

#endif // DUNE_UDGTEST_UTILITY_VTK_HH
