#!/usr/bin/env python

from sys import exit

from lut.vtk import Vtk
from lut.vtk import VtkFaces
from lut.generator import LookupGenerator
#from lut.generator import GeneratorContainer
from lut.consistencycheck import Consistency
from lut.dunecode import DuneCode
from lut.sk import Sk

########################
def getPos(v):
	pos = [ (x,y,z) for z in [0,1] for y in [0,1] for x in [0,1]]
	if type(v) is int:
		return pos[v]
	else:
		return tuple( 0.5*( pos[v[0]][d] + pos[v[1]][d] ) for d in [0,1,2] )

class Vector:
	def __init__(self, data):
		if type(data) in [tuple, list] and len(data) == 3:
			self.data = list(data)
		else:
			self.data = getPos(data)
	def __repr__(self):
		return repr(self.data)  
	def __add__(self, other):
		data = []
		for j in range(len(self.data)):
			data.append(self.data[j] + other.data[j])
		return Vector(data)
	def __sub__(self, other):
		data = []
		for j in range(len(self.data)):
			data.append(self.data[j] - other.data[j])
		return Vector(data)
	def crossProd(self, other):
		return Vector( [ (self.data[(d+1)%3]*other.data[(d+2)%3]
						  - self.data[(d+2)%3]*other.data[(d+1)%3]) for d in [0,1,2] ] )
	def __mul__(self, other):
		return sum ( [ self.data[d]*other.data[d] for d in [0,1,2] ] )

def checkVol3Dx(vertices):
	if len(vertices) == 4:
		a = Vector(vertices[0])
		b = Vector(vertices[1])
		c = Vector(vertices[2])
		d = Vector(vertices[3])
		vol = ((b-a).crossProd(c-a))*(d-a)
		assert(vol>0)
	if len(vertices) == 5:
		a = Vector(vertices[0])
		b = Vector(vertices[1])
		c = Vector(vertices[3])
		d = Vector(vertices[4])
		vol = ((b-a).crossProd(c-a))*(d-a)
		assert(vol>=0)
	if len(vertices) == 6:
		a = Vector(vertices[0])
		b = Vector(vertices[1])
		c = Vector(vertices[2])
		d = Vector(vertices[3])
		e = Vector(vertices[4])
		f = Vector(vertices[5])
		for V in [(d-a), (e-b), (f-c)]:
			vol1 = ((b-a).crossProd(c-a))*V
			vol2 = ((e-d).crossProd(f-d))*V
			assert(vol1>0)
			assert(vol2>0)
	if len(vertices) == 8:
		a = Vector(vertices[0])
		b = Vector(vertices[1])
		c = Vector(vertices[2])
		d = Vector(vertices[4])
		e = Vector(vertices[5])
		f = Vector(vertices[6])
		for V in [(d-a), (e-b), (f-c)]:
			vol1 = ((b-a).crossProd(c-a))*V
			vol2 = ((e-d).crossProd(f-d))*V
			assert(vol1>0)
			assert(vol2>0)
	else:
		pass

def checkVol3D(cases):
	for case in cases.base_cases:
		for cell in case.cells:
			try:
				checkVol3Dx(cell)
			except AssertionError:
				print "Error: BaseCase[%i]: Cell %i %s" % \
					  (cases.base_cases.index(case), case.cells.index(cell), repr(cell))

################################################################################
## 3D Cube                                                                    ##
################################################################################
cube3d = LookupGenerator(3,"cube")
# base cases cube 3D:
# 0,0,0,0,0,0,0,0 -> 00000000
cube3d.base_cases[0].faces = [[]]
cube3d.base_cases[0].cells = [[0, 1, 2, 3, 4, 5, 6, 7]]
# 1,0,0,0,0,0,0,0 -> 00000001
cube3d.base_cases[1].faces = [[(0, 4), (0, 1), (0, 2)]]
cube3d.base_cases[1].cells = [[(0, 4), (0, 1), (0, 2), 4, 5, 6], [(0, 1), 5, 1, (0, 2), 6, 2], [1, 3, 2, 5, 7, 6]]
# 1,1,0,0,0,0,0,0 -> 00000011
cube3d.base_cases[2].faces = [[(0, 2), (1, 3), (0, 4), (1, 5)]]
cube3d.base_cases[2].cells = [[(1, 3), (1, 5), 3, 7, (0, 2), (0, 4), 2, 6], [(1, 5), 5, 7, (0, 4), 4, 6]]
# 0,1,1,0,0,0,0,0 -> 00000110
cube3d.base_cases[3].faces = [[(1, 5), (1, 3), (0, 1)], [(2, 3), (0, 2), (2, 6)]]
cube3d.base_cases[3].cells = [[(1, 5), (1, 3), (0, 1), 5, 7, 4], [(0, 2), (2, 3), (2, 6), 4, 7, 6], [3, (2, 3), (0, 1), 7], [3, (0, 1), (1, 3), 7], [0, (0, 1), (2, 3), 4], [0, (2, 3), (0, 2), 4], [(0, 1), (2, 3), 4, 7]]
# 1,1,1,0,0,0,0,0 -> 00000111
cube3d.base_cases[4].faces = [[(1, 3), (2, 3), (1, 5), (2, 6)], [(1, 5), (2, 6), (0, 4)]]
cube3d.base_cases[4].cells = [[(1, 5), (2, 6), (0, 4), 5, 6, 4], [3, (2, 3), (1, 3), 7, 6, 5], [(2, 6), (2, 3), 6, (1, 5), (1, 3), 5]]
# 1,1,1,1,0,0,0,0 -> 00001111
cube3d.base_cases[5].faces = [[(0, 4), (1, 5), (2, 6), (3, 7)]]
cube3d.base_cases[5].cells = [[(0, 4), (1, 5), (2, 6), (3, 7), 4, 5, 6, 7]]
# 0,1,1,0,1,0,0,0 -> 00010110
cube3d.base_cases[6].faces = [[(0, 1), (1, 3), (1, 5)], [(0, 2), (2, 3), (2, 6)], [(0, 4), (4, 5), (4, 6)]]
cube3d.base_cases[6].cells = [[(0, 1), (1, 5), (1, 3), 0, 5, 3], [(4, 5), (0, 4), (4, 6), 5, 0, 6], [(2, 3), (2, 6), (0, 2), 3, 6, 0], [5, 6, 3, 0], [5, 3, 6, 7]]
# 1,1,1,0,1,0,0,0 -> 00010111
cube3d.base_cases[7].faces = [[(2, 3), (1, 3), (2, 6), (1, 5)], [(2, 6), (1, 5), (4, 6), (4, 5)]]
cube3d.base_cases[7].cells = [[(2, 3), (1, 3), 3, 6, 5, 7], [6, (2, 6), (2, 3), 5, (1, 5), (1, 3)], [(4, 6), (2, 6), 6, (4, 5), (1, 5), 5]]
# 0,0,0,1,1,0,0,0 -> 00011000
cube3d.base_cases[8].faces = [[(0, 4), (4, 6), (4, 5)], [(1, 3), (2, 3), (3, 7)]]
cube3d.base_cases[8].cells = [[(2, 3), (1, 3), (3, 7), 6, 5, 7], [1, 2, 0, (4, 5), (4, 6), (0, 4)], [(2, 3), 6, 2, (1, 3), 5, 1], [2, 6, (4, 6), 1, 5, (4, 5)]]
# 1,0,0,1,1,0,0,0 -> 00011001
cube3d.base_cases[9].faces = [[(4, 5), (4, 6), (0, 1), (0, 2)], [(2, 3), (3, 7), (1, 3)]]
cube3d.base_cases[9].cells = [[(4, 6), (4, 5), (0, 2), (0, 1), 7, 5, (1, 3), 1], [(4, 6), 6, 7, (0, 2), 2, (1, 3)], [6, 7, (1, 3), (3, 7)], [6, (1, 3), 2, (3, 7)], [2, (3, 7), (1, 3), (2, 3)]];
# 1,1,0,1,1,0,0,0 -> 00011011
cube3d.base_cases[10].faces = [[(6, 4), (4, 5), (2, 0)], [(4, 5), (7, 3), (2, 0)], [(4, 5), (7, 3), (5, 1)], [(7, 3), (2, 0), (2, 3)]]
cube3d.base_cases[10].cells = [[(6, 4), (4, 5), (2, 0), (7, 3)], [6, (6, 4), (2, 0), (7, 3)], [6, (2, 0), 2, (7, 3)], [(7, 3), (2, 0), 2, (2, 3)], [7, (4, 5), (6, 4), (7, 3)], [7, (6, 4), 6, (7, 3)], [7, (7, 3), (5, 1), (4, 5)], [7, (5, 1), 5, (4, 5)]]
# 0,1,1,1,1,0,0,0 -> 00011110
cube3d.base_cases[11].faces = [[(0, 1), (0, 2), (1, 5), (2, 6)], [(0, 4), (4, 5), (4, 6)], [(1, 5), (2, 6), (3, 7)]]
cube3d.base_cases[11].cells = [[0, (0, 1), (0, 2), (0, 4), (4, 5), (4, 6)], [(0, 2), (0, 1), (2, 6), (1, 5), (4, 6), (4, 5), 6, 5], [(2, 6), (1, 5), (3, 7), 6, 5, 7]]
# 1,1,1,1,1,0,0,0 -> 00011111
cube3d.base_cases[12].faces = [[(4, 5), (4, 6), (1, 5), (2, 6)], [(1, 5), (2, 6), (3, 7)]]
cube3d.base_cases[12].cells = [[(4, 5), 5, (1, 5), (4, 6), 6, (2, 6)], [6, 7, 5, (2, 6), (3, 7), (1, 5)]]
# 0,0,1,1,1,1,0,0 -> 00111100
cube3d.base_cases[13].faces = [[(0, 2), (1, 3), (2, 6), (3, 7)], [(0, 4), (1, 5), (4, 6), (5, 7)]]
cube3d.base_cases[13].cells = [[0, 1, (0, 2), (1,3), 6, 7, (2, 6), (3, 7)], [(4, 6), (0, 4), 6, 0, (5, 7), (1, 5), 7, 1]]
# 1,0,1,1,1,1,0,0 -> 00111101
cube3d.base_cases[14].faces = [[(1, 5), (5, 7), (4, 6)], [(0, 1), (1, 5), (4, 6)], [(2, 6), (0, 1), (4, 6)], [(0, 1), (1, 3), (2, 6), (3, 7)]]
cube3d.base_cases[14].cells = [[6, (4, 6), (2, 6), 7, (5, 7), (3, 7)], [(0, 1), (1, 5), (5, 7), (4, 6)], [(0, 1), (5, 7), (2, 6), (4, 6)], [(0, 1), (1, 5), (1, 3), (2, 6), (5, 7), (3, 7)], [(0, 1), 1, (1, 3), (1, 5)]]
# 1,1,1,1,1,1,0,0 -> 00111111
cube3d.base_cases[15].faces = [[(2, 6), (4, 6), (3, 7), (5, 7)]]
cube3d.base_cases[15].cells = [[6, (4, 6), (2, 6), 7, (5, 7), (3, 7)]]
# 1,0,0,1,0,1,1,0 -> 01101001
cube3d.base_cases[16].faces = [[(0, 1), (0, 2), (0, 4)], [(2, 3), (1, 3), (3, 7)], [(4, 5), (5, 7), (1, 5)], [(2, 6), (4, 6), (6, 7)]]
cube3d.base_cases[16].cells = [[(0, 4), (0, 1), (0, 2), 4, (4, 5), (4, 6)], [(4, 5), (5, 7), (1, 5), (0, 1), (1, 3), 1], [(1, 3), (3, 7), (2, 3), (5, 7), 7, (6, 7)], [(2, 6), (6, 7), (4, 6), 2, (2, 3), (0, 2)], [(0, 1), (1, 3), (0, 2), (2, 3), (4, 5), (5, 7), (4, 6), (6, 7)]]
# 1,1,0,1,0,1,1,0 -> 01101011
cube3d.base_cases[17].faces = [[(4, 6), (6, 7), (2, 6)], [(0, 2), (2, 3), (0, 4), (4, 5)], [(2, 3), (3, 7), (4, 5), (5, 7)]]
cube3d.base_cases[17].cells = [[(6, 7), (4, 6), (2, 6), 7, (5, 7), (3, 7)], [(2, 3), (4, 5), (3, 7), (5, 7), 2, 4, (2, 6), (4, 6)], [2, (0, 2), (2, 3), 4, (0, 4), (4, 5)]]
# 1,1,1,1,0,1,1,0 -> 01101111
cube3d.base_cases[18].faces = [[(0, 4), (3,7), (4, 5), (5, 7)], [(0, 4), (3,7), (4, 6), (6, 7)]]
cube3d.base_cases[18].cells = [[4, (4, 5), (0, 4), (4, 6)], [(4, 6), (4, 5), (0, 4), (6, 7), (5, 7), (3, 7)], [(6, 7), 7, (5, 7), (3, 7)]]
# 0,1,1,1,1,1,1,0 -> 01111110
cube3d.base_cases[19].faces = [[(0, 4), (6, 7), (0, 2), (3, 7)], [(0, 1), (5, 7), (0, 4), (6, 7)], [(0, 2), (3, 7), (0, 1), (5, 7)]]
cube3d.base_cases[19].cells = [[0, (0, 1), (0, 2), (0, 4)], [(0, 4), (0, 1), (0, 2), (6, 7), (5, 7), (3, 7)], [(6, 7), 7, (5, 7), (3, 7)]]
# 1,1,1,1,1,1,1,0 -> 01111111
cube3d.base_cases[20].faces = [[(6, 7), (5, 7), (3, 7)]]
cube3d.base_cases[20].cells = [[(6, 7), (5, 7), (3, 7), 7]]
# 1,1,1,1,1,1,1,1 -> 11111111
cube3d.base_cases[21].faces = [[]]
cube3d.base_cases[21].cells = [[]]
# generate code
cube3d.generate()

checkVol3D(cube3d)

def printInverse(i):
	j = int("".join([repr(y) for y in cube3d.base_cases[i].case[::-1]]), 2)
	k = 255 ^ j
	#print cube3d.base_cases[i].case
	#print cube3d.all_cases[j].case
	#print cube3d.all_cases[k].case
	#print cube3d.all_cases[k].base_case.case
	m = cube3d.base_cases.index(cube3d.all_cases[k].base_case)
	#print cube3d.base_cases[m].case
	print "inverse(%i) -> %i" %(i, m)

printInverse(14)
printInverse(17)
printInverse(18)
printInverse(19)

################################################################################
## 3D Simplex                                                                 ##
################################################################################
simplex3d = LookupGenerator(3,"simplex")
# base cases simplex 3D:
# 0,0,0,0 -> 0000
simplex3d.base_cases[0].faces = [[]]
simplex3d.base_cases[0].cells = [[0, 1, 2, 3]]
# 1,0,0,0 -> 0001
simplex3d.base_cases[1].faces = [[(0,2), (0,1), (0,3)]]
simplex3d.base_cases[1].cells = [[(0,1), (0,2), (0,3), 1, 2, 3]]
# 1,1,0,0 -> 0011
simplex3d.base_cases[2].faces = [[(0,2), (1,2), (0,3), (1,3)]]
simplex3d.base_cases[2].cells = [[(0,2), (1,2), 2, (0,3), (1,3), 3]]
# 1,1,1,0 -> 0111
simplex3d.base_cases[3].faces = [[(1,3), (0,3), (2,3)]]
simplex3d.base_cases[3].cells = [[(0,3), (1,3), (2,3), 3]]
# 1,1,1,1 -> 1111
simplex3d.base_cases[4].faces = [[]]
simplex3d.base_cases[4].cells = [[]]
# generate code
simplex3d.generate()

################################################################################
## 2D Cube                                                                    ##
################################################################################
cube2d = LookupGenerator(2,"cube")
# base cases cube 2D:
# 0,0,0,0 -> 0000
cube2d.base_cases[0].faces = [[], []]
cube2d.base_cases[0].cells = [[0, 1, 2, 3], [], []]
# 1,0,0,0 -> 0001
cube2d.base_cases[1].faces = [[(0, 1), (0, 2)], []]
cube2d.base_cases[1].cells = [[3, 2, (0, 2)], [3, (0, 2), (0, 1)], [3, (0, 1), 1]]
# 1,1,0,0 -> 0011
cube2d.base_cases[2].faces = [[(1, 3), (0, 2)], []]
cube2d.base_cases[2].cells = [[(0, 2), (1, 3), 2, 3], [], []]
# 0,1,1,0 -> 0110
cube2d.base_cases[3].faces = [[(0, 1), (1, 3)], [(0, 2), (2, 3)]]
cube2d.base_cases[3].cells = [[0, 3, (0, 2), (2, 3)], [(0,1), (1,3), 0, 3], []]
# 1,1,1,0 -> 0111
cube2d.base_cases[4].faces = [[(2, 3), (1, 3)], []]
cube2d.base_cases[4].cells = [[3, (2, 3), (1, 3)], [], []]
# 1,1,1,1 -> 1111
cube2d.base_cases[5].faces = [[], []]
cube2d.base_cases[5].cells = [[], [], []]
# generate code
cube2d.generate()

################################################################################
## 2D Simplex                                                                 ##
################################################################################
simplex2d = LookupGenerator(2,"simplex")
# base cases simplex 2D:
# 0,0,0 -> 000
simplex2d.base_cases[0].faces = [[]]
simplex2d.base_cases[0].cells = [[0, 1, 2]]
# 1,0,0 -> 001
simplex2d.base_cases[1].faces = [[(0,1), (0,2)]]
simplex2d.base_cases[1].cells = [[(0,2), (0,1), 2, 1]]
# 1,1,0 -> 011
simplex2d.base_cases[2].faces = [[(0,2), (1,2)]]
simplex2d.base_cases[2].cells = [[(0,2), (1,2), 2]]
# 1,1,1 -> 111
simplex2d.base_cases[3].faces = [[]]
simplex2d.base_cases[3].cells = [[]]
# generate code
simplex2d.generate()

################################################################################
## 1D Cube                                                                    ##
################################################################################
lut1d = LookupGenerator(1,"any")
# base cases cube 1D:
# 0,0 -> 00
lut1d.base_cases[0].faces = [[]]
lut1d.base_cases[0].cells = [[0,1]]
# 1,0 -> 01
lut1d.base_cases[1].faces = [[(0,1)]]
lut1d.base_cases[1].cells = [[(0,1), 1]]
# 1,1 -> 11
lut1d.base_cases[2].faces = [[]]
lut1d.base_cases[2].cells = [[]]
# generate code
lut1d.generate()


################################################################################
## 0D Cube                                                                    ##
################################################################################
lut0d = LookupGenerator(0,"any")
# base cases cube 0D:
# 0 -> 0
lut0d.base_cases[0].faces = [[]]
lut0d.base_cases[0].cells = [[0]]
# 1 -> 1
lut0d.base_cases[1].faces = [[]]
lut0d.base_cases[1].cells = [[]]
# generate code
lut0d.generate()

################################################################################
## Output                                                                     ##
################################################################################

#print """namespace Dune {
#
#    namespace MarchingInternal {
#"""

ccfile = open("marchinglut.cc", "w")
ccfile.write("""
        /*
         * This file is autogenerated using generate_lut.py
         *
         * Don't edit this file!
         */

        /* lookuptable interface */
        template<int G, int D>
        class P1Lut;\n""")
DuneCode(cube3d).write(ccfile)
DuneCode(simplex3d).write(ccfile)
DuneCode(cube2d).write(ccfile)
DuneCode(simplex2d).write(ccfile)
DuneCode(lut1d).write(ccfile)
DuneCode(lut0d).write(ccfile)
ccfile.close()

Vtk(cube3d).write("lut/vtk")
VtkFaces(cube3d).write("lut/vtk/faces")
Vtk(simplex3d).write("lut/vtk")
Vtk(cube2d).write("lut/vtk")
Vtk(simplex2d).write("lut/vtk")

generators = {
	(2,"simplex"): simplex2d,
	(2,"cube"): cube2d,
	(3,"simplex"): simplex3d,
#	(3,"prism"): prism3d,
#	(3,"pyramid"): pyramid3d,
	(3,"cube"): cube3d
	}

Consistency(generators).check(3, "simplex")
#Consistency(generators).check(3, "cube")

Sk(cube3d).write("lut/sk")
Sk(simplex3d).write("lut/sk")
