// -*- tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*-
// vi: set et ts=4 sw=2 sts=2:
#include <config.h>

#include <dune/udg/common/subentity.hh>
#include <dune/udg/2dtriangulation/innerdomain.hh>

#include <dune/grid/yaspgrid.hh>
#if HAVE_UG
#include <dune/grid/uggrid.hh>
#endif
#if HAVE_ALUGRID
#include <dune/grid/alugrid.hh>
#endif

namespace Dune {
  namespace UDG {

    //////////////////////////////////////////////////////////////////////////
    // Instantiation
    //////////////////////////////////////////////////////////////////////////

    // -- 2d --

    template
    EntityPart< Dune::YaspGrid<2> >::EntityPart(const BoundingBoxGeometry & _r,
                                                const Point & A, const Point & B, const Point & C, int tag, int index);

    template
    EntityPart< Dune::YaspGrid<2> >::EntityPart(const BoundingBoxGeometry & _r,
                                                const Point & A, const Point & B, const Point & C,
                                                const InnerDomain<2,double> & i, int tag, int index);
    template
    EntityPart< Dune::YaspGrid<2> >::EntityPart(const BoundingBoxGeometry & _r, int tag, int index);

    template
    EntityPart< Dune::YaspGrid<2> >::EntityPart(const BoundingBoxGeometry & _r,
                                                const Point & A, const Point & B, int tag, int index);
    template
    EntityPart< Dune::YaspGrid<2> >::EntityPart(const BoundingBoxGeometry & _r,
                                                const Point & A, const Point & B,
                                                const Point & C, const Point & D, int tag, int index);

    // -- 3d --

    template
    EntityPart< Dune::YaspGrid<3> >::EntityPart(const BoundingBoxGeometry & _r, int tag, int index);

    template
    EntityPart< Dune::YaspGrid<3> >::EntityPart(const BoundingBoxGeometry & _r,
                                                const Point & A, const Point & B, int tag, int index);

    template
    EntityPart< Dune::YaspGrid<3> >::EntityPart(const BoundingBoxGeometry & _r,
                                                const Point & A, const Point & B,
                                                const Point & C, const Point & D,
                                                const Point & E, const Point & F,
                                                const Point & G, const Point & H, int tag, int index);

    template
    EntityPart< Dune::YaspGrid<3> >::EntityPart(const BoundingBoxGeometry & _r,
                                                const Point & A, const Point & B,
                                                const Point & C, const Point & D,
                                                const Point & E, const Point & F, int tag, int index);

    template
    EntityPart< Dune::YaspGrid<3> >::EntityPart(const BoundingBoxGeometry & _r,
                                                const Point & A, const Point & B,
                                                const Point & C, const Point & D,
                                                const Point & E, int tag, int index);

    template
    EntityPart< Dune::YaspGrid<3> >::EntityPart(const BoundingBoxGeometry & _r,
                                                const Point & A, const Point & B,
                                                const Point & C, const Point & D, int tag, int index);

#if HAVE_UG
    // -- 2d --

    template
    EntityPart< Dune::UGGrid<2> >::EntityPart(const BoundingBoxGeometry & _r,
                                              const Point & A, const Point & B, const Point & C, int tag, int index);
    template
    EntityPart< Dune::UGGrid<2> >::EntityPart(const BoundingBoxGeometry & _r,
                                              const Point & A, const Point & B, const Point & C,
                                              const InnerDomain<2,double> & i, int tag, int index);
    template
    EntityPart< Dune::UGGrid<2> >::EntityPart(const BoundingBoxGeometry & _r, int tag, int index);

    template
    EntityPart< Dune::UGGrid<2> >::EntityPart(const BoundingBoxGeometry & _r,
                                              const Point & A, const Point & B, int tag, int index);
    template
    EntityPart< Dune::UGGrid<2> >::EntityPart(const BoundingBoxGeometry & _r,
                                              const Point & A, const Point & B,
                                              const Point & C, const Point & D, int tag, int index);

    // -- 3d --

    template
    EntityPart< Dune::UGGrid<3> >::EntityPart(const BoundingBoxGeometry & _r, int tag, int index);

    template
    EntityPart< Dune::UGGrid<3> >::EntityPart(const BoundingBoxGeometry & _r,
                                              const Point & A, const Point & B, int tag, int index);

    template
    EntityPart< Dune::UGGrid<3> >::EntityPart(const BoundingBoxGeometry & _r,
                                              const Point & A, const Point & B,
                                              const Point & C, const Point & D,
                                              const Point & E, const Point & F,
                                              const Point & G, const Point & H, int tag, int index);

    template
    EntityPart< Dune::UGGrid<3> >::EntityPart(const BoundingBoxGeometry & _r,
                                              const Point & A, const Point & B,
                                              const Point & C, const Point & D,
                                              const Point & E, const Point & F, int tag, int index);

    template
    EntityPart< Dune::UGGrid<3> >::EntityPart(const BoundingBoxGeometry & _r,
                                              const Point & A, const Point & B,
                                              const Point & C, const Point & D,
                                              const Point & E, int tag, int index);

    template
    EntityPart< Dune::UGGrid<3> >::EntityPart(const BoundingBoxGeometry & _r,
                                              const Point & A, const Point & B,
                                              const Point & C, const Point & D, int tag, int index);
#endif

#if HAVE_ALUGRID
    // -- 3d --

    template
    EntityPart< Dune::ALUGrid< 3, 3, cube, nonconforming > >::EntityPart(const BoundingBoxGeometry & _r, int tag, int index);

    template
    EntityPart< Dune::ALUGrid< 3, 3, cube, nonconforming > >::EntityPart(const BoundingBoxGeometry & _r,
                                                     const Point & A, const Point & B, int tag, int index);

    template
    EntityPart< Dune::ALUGrid< 3, 3, cube, nonconforming > >::EntityPart(const BoundingBoxGeometry & _r,
                                                     const Point & A, const Point & B,
                                                     const Point & C, const Point & D,
                                                     const Point & E, const Point & F,
                                                     const Point & G, const Point & H, int tag, int index);

    template
    EntityPart< Dune::ALUGrid< 3, 3, cube, nonconforming > >::EntityPart(const BoundingBoxGeometry & _r,
                                                     const Point & A, const Point & B,
                                                     const Point & C, const Point & D,
                                                     const Point & E, const Point & F, int tag, int index);

    template
    EntityPart< Dune::ALUGrid< 3, 3, cube, nonconforming > >::EntityPart(const BoundingBoxGeometry & _r,
                                                     const Point & A, const Point & B,
                                                     const Point & C, const Point & D,
                                                     const Point & E, int tag, int index);

    template
    EntityPart< Dune::ALUGrid< 3, 3, cube, nonconforming > >::EntityPart(const BoundingBoxGeometry & _r,
                                                     const Point & A, const Point & B,
                                                     const Point & C, const Point & D, int tag, int index);

    // -- 2d --

    template
    EntityPart< Dune::ALUGrid< 2, 2, simplex, nonconforming > >::EntityPart(const BoundingBoxGeometry & _r, int tag, int index);

    template
    EntityPart< Dune::ALUGrid< 2, 2, simplex, nonconforming > >::EntityPart(const BoundingBoxGeometry & _r,
                                                        const Point & A, const Point & B, const Point & C, int tag, int index);
#endif

  } // end UDG
} // end Dune
